void tsp_3opt_limited(int n,                                 // Number of cities
                      int** d,                                // Distance matrix
                      int r,                                  // Subproblem size
                      int succ[],                 //InOut Successor of each city
                      int* length)                          // InOut Tour length
{ if (r > n)                           // Max subpath length should not exceed n
    r = n ;
  int i = 0, last_i = 0;                 // First subpath starts from city i = 0
  do
  { int edges_ij = 1, edges_jk = 1;   // Nr of edges from i to j and from j to k
    for (int j = succ[i]; edges_ij + edges_jk < r; 
         j = succ[j], ++edges_ij, edges_jk = 1)
    { for (int k = succ[j]; edges_ij + edges_jk < r; k = succ[k], ++edges_jk)
      { int delta = d[i][succ[j]] + d[j][succ[k]] + d[k][succ[i]]   // Move cost
                   -d[i][succ[i]] - d[j][succ[j]] - d[k][succ[k]];
        if (delta < 0)                                       // Improving move ?
        { *length += delta;                              // Update solution cost
          int tmp = succ[i];  succ[i] = succ[j]; 
          succ[j] = succ[k]; succ[k] = tmp;                      // Perform move
          tmp = j; j = k; k = tmp;                  // Replace j between i and k
          tmp = edges_ij; edges_ij = edges_jk; edges_jk = tmp;
          last_i = i;                           // Update last values of index i
        }
      }
    }
    i = succ[i];
  }
  while (i != last_i);            // A complete tour scanned without improvement
} // tsp_3opt_limited
