void tsp_SA(int n,                                           // Number of cities
            int** d,                     // Distance matrix, must be symmetrical
            int best_tour[],                                   // InOut TSP tour
            int* best_length,                   // InOut Length of the best tour
            double initial_temperature,                         // SA Parameters
            double final_temperature,
            double alpha)

{ int* tour = (int*) malloc((size_t)n * sizeof(int));            // Current tour
  memcpy(tour, best_tour, (size_t)n * sizeof(int));   
  int length = *best_length;                           // Length of current tour
  double T = initial_temperature;                         // Current temperature
  long iteration = 0;                          // Number of iterations performed
  while (T > final_temperature)         // Repeat, while temperature high enough
  { int i = unif(0, n - 1),              // First city of a move randomly chosen
        j = (i + unif(2, n - 2))%n;    // Second city is unif successors further
    if (j < i)
      swap(&i, &j);                             // j must be further on the tour
    int delta =  d[tour[i]][tour[j]]     + d[tour[i + 1]][tour[(j + 1) % n]] -
                 d[tour[i]][tour[i + 1]] - d[tour[j]][tour[(j + 1) % n]];
    if (delta < 0 || exp(-delta/T) > rando())                   // Move accepted
    { length = length + delta;
      for (int k = 0; k < (j - i)/2; ++k)
        swap(tour + i+k+1, tour + j-k);      // Reverse sub-path between i and j
     
      if (length < *best_length)                 // Store improved best solution
      { *best_length = length;
         memcpy(best_tour, tour, (size_t)n * sizeof(int));
         printf("SA iteration: %ld, length: %d\n", iteration, length);
      }
    } // Move accepted 
    ++iteration;
    if (iteration % (n*n) == 0)     // Temperature decreases every n*n iteration
      T *= alpha;
  } // while (T > final_temperature)
  free(tour);
} // tsp_SA
